@@ -12,8 +12,10 @@ These sockets allow nonblocking event-driven operation:
 
 History:
 2012-08-10 ROwen    Based on TkSocket.
+2014-04-10 ROwen    Added NullTCPSocket and added name argument ot NullSocket.
+                    Removed duplicate definition of BaseServer._basicClose.
 """
-__all__ = ["BaseSocket", "BaseServer", "NullSocket", "nullCallback"]
+__all__ = ["BaseSocket", "BaseServer", "NullSocket", "NullTCPSocket", "nullCallback"]
 
 import sys
 import traceback
@@ -34,18 +36,22 @@ class Base(object):
     """
     def __init__(self,
         state,
-        stateCallback = nullCallback,
+        stateCallback = None,
         name = "",
     ):
         """Construct a Base
         
         @param[in] state: initial state
-        @param[in] stateCallback: function to call when socket state changes; it receives one argument: this socket
+        @param[in] stateCallback: function to call when socket state changes; it receives one argument: this socket;
+            if None then no callback
         @param[in] name: a string to identify this object; strictly optional
         """
         self._state = state
         self._reason = ""
-        self._stateCallback = stateCallback
+        if stateCallback:
+            self._stateCallbackList = [stateCallback]
+        else:
+            self._stateCallbackList = []
         self.name = name
 
     @property
@@ -74,14 +80,50 @@ class Base(object):
         """
         return self._state in self._ReadyStates
     
-    def setStateCallback(self, callFunc=nullCallback):
-        """Set the state callback function (replacing the current one).
+    @property
+    def didFail(self):
+        """Return True if object failed, e.g. connection failed
+        """
+        return self._state in self._FailedStates
+    
+    def addStateCallback(self, callFunc):
+        """Add a state callback function
+        """
+        self._stateCallbackList.append(callFunc)
+
+    def removeStateCallback(self, callFunc, doRaise=False):
+        """Delete the callback function.
+
+        Inputs:
+        - callFunc  callback function to remove
+        - doRaise   raise an exception if unsuccessful? True by default.
+
+        Return:
+        - True if successful, raise error or return False otherwise.
+        
+        If doRaise true and callback not found then raise ValueError.
+        """
+        try:
+            self._stateCallbackList.remove(callFunc)
+            return True
+        except ValueError:
+            if doRaise:
+                raise ValueError("Callback %r not found" % callFunc)
+            return False
+    
+    def setStateCallback(self, callFunc=None):
+        """Set the state callback function (replacing all current ones).
+        
+        Deprecated; please use addStateCallback instead.
         
         Inputs:
-        - callFunc: the callback function, or nullCallback if none wanted
+        - callFunc: the callback function, or None if none wanted
                     The function is sent one argument: this Socket
         """
-        self._stateCallback = callFunc
+        if callFunc:
+            self._stateCallbackList = [callFunc]
+        else:
+            self._stateCallbackList = []
     
     def setName(self, newName):
         """Set socket name
@@ -91,7 +133,7 @@ class Base(object):
     def _clearCallbacks(self):
         """Clear any callbacks added by this class.
         """
-        self._stateCallback = nullCallback
+        self._stateCallbackList = []
 
     def _setState(self, newState, reason=None):
         """Change the state.
@@ -100,7 +142,7 @@ class Base(object):
         - newState: the new state
         - reason: an explanation (None to leave alone)
         """
-#         print "%s._setState(newState=%s, reason=%s)" % (newState, reason)
+        # print "%s._setState(newState=%s, reason=%s)" % (self, newState, reason)
         if self.isDone:
             raise RuntimeError("Already done; cannot change state")
 
@@ -108,26 +150,26 @@ class Base(object):
         if reason is not None:
             self._reason = str(reason)
         
-        stateCallback = self._stateCallback
+        for stateCallback in self._stateCallbackList:
+            try:
+                stateCallback(self)
+            except Exception, e:
+                sys.stderr.write("%s state stateCallback %s failed: %s\n" % (self, stateCallback, e,))
+                traceback.print_exc(file=sys.stderr)
+
         if self.isDone:
             try:
                 self._clearCallbacks()
             except Exception, e:
                 sys.stderr.write("%s failed to clear callbacks: %s\n" % (self, e,))
                 traceback.print_exc(file=sys.stderr)
-        
-        try:
-            stateCallback(self)
-        except Exception, e:
-            sys.stderr.write("%s state callback %s failed: %s\n" % (self, self._stateCallback, e,))
-            traceback.print_exc(file=sys.stderr)
 
     def _getArgStr(self):
-        """Return main arguments as a string, for __str__
+        """Return main arguments as a string, for __repr__
         """
         return "name=%r" % (self.name)
 
-    def __str__(self):
+    def __repr__(self):
         return "%s(%s)" % (self.__class__.__name__, self._getArgStr())
 
 
@@ -151,6 +193,7 @@ class BaseSocket(Base):
     ))
     _DoneStates = set((Closed, Failed))
     _ReadyStates = set((Connected,))
+    _FailedStates = set((Failed,))
         
     StateStrMaxLen = 0
     for _stateStr in _AllStates:
@@ -159,8 +202,8 @@ class BaseSocket(Base):
     
     def __init__(self,
         state = Connected,
-        readCallback = nullCallback,
-        stateCallback = nullCallback,
+        readCallback = None,
+        stateCallback = None,
         name = "",
     ):
         """Construct a BaseSocket
@@ -171,7 +214,7 @@ class BaseSocket(Base):
         - stateCallback: function to call when socket state changes; it receives one argument: this socket
         - name: a string to identify this socket; strictly optional
         """
-        self._readCallback = readCallback
+        self._readCallback = readCallback or nullCallback
         Base.__init__(self,
             state = state,
             stateCallback = stateCallback,
@@ -196,14 +239,14 @@ class BaseSocket(Base):
         """
         raise NotImplementedError()
     
-    def setReadCallback(self, callFunc=nullCallback):
+    def setReadCallback(self, callFunc=None):
         """Set the read callback function (replacing the current one).
         
         Inputs:
         - callFunc: the callback function, or nullCallback if none wanted.
                     The function is sent one argument: this Socket
         """
-        self._readCallback = callFunc
+        self._readCallback = callFunc or nullCallback
     
     def write(self, data):
         """Write data to the socket. Does not block.
@@ -234,6 +277,8 @@ class BaseSocket(Base):
         - isOK: if True, mark state as Closed, else Failed
         - reason: a string explaining why, or None to leave unchanged;
             please specify if isOK is false.
+
+        The twisted version returns a Deferred if the socket is not already closed, else None
         """
 #         print "%s.close(isOK=%r, reason=%r)" % (self, isOK, reason)
         if self.isDone:
@@ -278,13 +323,14 @@ class BaseServer(Base):
     ))
     _DoneStates = set((Closed, Failed))
     _ReadyStates = set((Listening,))
+    _FailedStates = set((Failed,))
 
     def __init__(self,
         connCallback,
         state = Starting,
-        stateCallback = nullCallback,
-        sockReadCallback = nullCallback,
-        sockStateCallback = nullCallback,
+        stateCallback = None,
+        sockReadCallback = None,
+        sockStateCallback = None,
         name = "",
     ):
         """Construct a socket server
@@ -298,19 +344,14 @@ class BaseServer(Base):
         - name: a string to identify this server; strictly optional
         - state: initial state
         """
-        self._connCallback = connCallback
-        self._sockReadCallback = sockReadCallback
-        self._sockStateCallback = sockStateCallback
+        self._connCallback = connCallback or nullCallback
+        self._sockReadCallback = sockReadCallback or nullCallback
+        self._sockStateCallback = sockStateCallback or nullCallback
         Base.__init__(self,
             state = state,
             stateCallback = stateCallback,
             name = name,
         )        
-
-    def _basicClose(self):
-        """Shut down the server.
-        """
-        raise NotImplementedError()
     
     def close(self, isOK=True, reason=None):
         """Start closing the server.
@@ -321,6 +362,8 @@ class BaseServer(Base):
         - isOK: if True, mark state as Closed, else Failed
         - reason: a string explaining why, or None to leave unchanged;
             please specify if isOK is false.
+            
+        The twisted version returns a Deferred if the socket is not already closed, else None
         """
 #         print "%s.close(isOK=%r, reason=%r)" % (self, isOK, reason)
         if self.isDone:
@@ -331,7 +374,7 @@ class BaseServer(Base):
         else:
             self._setState(self.Failing, reason)
 
-        self._basicClose()
+        return self._basicClose()
 
     def _basicClose(self):
         """Start closing the socket.
@@ -353,8 +396,9 @@ class NullSocket(BaseSocket):
     Forbids read, write and setting a new state callback.
     Close is OK and the state is always Closed.
     """
-    def __init__ (self):
+    def __init__ (self, name=""):
         BaseSocket.__init__(self,
+            name = name,
             state = self.Closed,
         )
         self._reason = "This is an instance of NullSocket"
@@ -370,3 +414,15 @@ class NullSocket(BaseSocket):
 
     def writeLine(self, astr):
         raise RuntimeError("Cannot writeLine %r to null socket" % astr)
+
+
+class NullTCPSocket(NullSocket):
+    def __init__(self, name, host, port):
+        NullSocket.__init__(self, name)
+        self.host = host
+        self.port = port
+
+    def _getArgStr(self):
+        """Return main arguments as a string, for __repr__
+        """
+        return "name=%s, host=%s, port=%s" % (self.name, self.host, self.port)
