@@ -57,6 +57,14 @@ History:
 2007-01-11 ROwen    Added isDefault method.
 2010-05-21 ROwen    Added trackDefault parameter. By default this is set to autoIsCurrent,
                     so this may alter existing code.
+2012-10-25 ROwen    If width is specified, increase it on aqua to work around a Tk bug.
+2012-11-16 ROwen    Refine bug workaround to only occur if Tcl version starts with "8.5".
+2012-11-29 ROwen    Further refined bug workaround to make it work like unix as much as possible
+                    using default fonts, without getting fancy with font metrics.
+                    Fixed and enhanced the demo code, demo of fixed width.
+2012-11-30 ROwen    Bug fix: width patch was not applied if width changed after the widget was created.
+                    Now it is applied by overridden method configure.
+2012-11-30 ROwen    Does no width correction if bitmap is shown.
 """
 __all__ = ['Checkbutton']
 
@@ -91,7 +99,7 @@ class Checkbutton (Tkinter.Checkbutton, RO.AddCallback.TkVarMixin,
     - autoIsCurrent controls automatic isCurrent mode
         - if false (manual mode), then is/isn't current if set, setBool
             or setIsCurrent is called with isCurrent true/false
-        - if true (auto mode), then is currently only when all these are so:
+        - if true (auto mode), then isCurrent is true only when all these are so:
             - set, setBool or setIsCurrent is called with isCurrent true
             - setDefValue is called with isCurrent true
             - current value == default value
@@ -113,6 +121,9 @@ class Checkbutton (Tkinter.Checkbutton, RO.AddCallback.TkVarMixin,
       - text and textvariable are forbidden if showValue is true
       - selectcolor is ignored and forced equal to background if indicatoron false
         (i.e. if no checkbox is shown)
+        
+    Warning: as of Tcl/Tk 8.5 the indicatoron option is ignored on MacOS X (Aqua);
+    the checkbox is always displayed.
 
     Inherited methods include:
     addCallback, removeCallback
@@ -144,7 +155,7 @@ class Checkbutton (Tkinter.Checkbutton, RO.AddCallback.TkVarMixin,
             trackDefault = bool(autoIsCurrent)
         self._trackDefault = trackDefault
         self.helpText = helpText
-        
+
         # if a command is supplied in kargs, remove it now and set it later
         # so it is not called during init
         cmd = kargs.pop("command", None)
@@ -161,6 +172,7 @@ class Checkbutton (Tkinter.Checkbutton, RO.AddCallback.TkVarMixin,
         if not RO.CnvUtil.asBool(kargs.get("indicatoron", True)):
             # user wants text, not a checkbox;
             # on Aqua adjust default padding so text can be read
+            # also indicatoron is ignored on Aqua, except that it affects label width
             if RO.TkUtil.getWindowingSystem() == RO.TkUtil.WSysAqua:
                 kargs.setdefault("padx", 6)
                 kargs.setdefault("pady", 5)
@@ -170,7 +182,8 @@ class Checkbutton (Tkinter.Checkbutton, RO.AddCallback.TkVarMixin,
         Tkinter.Checkbutton.__init__(self,
             master = master,
             variable = self._var,
-        **kargs)
+        )
+        self.configure(kargs) # call overridden configure to fix width, if necessary
 
         RO.AddCallback.TkVarMixin.__init__(self, self._var)
         
@@ -345,43 +358,105 @@ class Checkbutton (Tkinter.Checkbutton, RO.AddCallback.TkVarMixin,
             self.configure(state="disabled")
             if self._defIfDisabled:
                 self.restoreDefault()
+    
+    def configure(self, argDict=None, **kargs):
+        """Overridden version of configure that applies a width correction, if necessary
+
+        Notes:
+        - configure is called by wdg[item] = value
+        - sometimes configure is called with a single positional argument: a dict of items,
+            and sometimes it is called with a set of keyword arguments. This code handles both cases.
+        - configure is NOT called by the widget's constructor, so you must call configure with your desired width
+            after constructing the widget, rather than passing width to the widget's constructor
+        """
+        if argDict is not None:
+            kargs.update(argDict)
+        if "width" in kargs:
+            kargs["width"] = self._computeCorrectedWidth(
+                width = kargs["width"],
+                hasBitmap = bool(kargs.get("bitmap", self["bitmap"])),
+                showIndicator = kargs.get("indicatoron", self["indicatoron"]),
+            )
+        Tkinter.Checkbutton.configure(self, **kargs)
+    
+    def _computeCorrectedWidth(self, width, hasBitmap, showIndicator):
+        """Compute corrected width to overcome Tcl/Tk bugs
+        """
+        if (width != 0) \
+            and not hasBitmap \
+            and (RO.TkUtil.getWindowingSystem() == RO.TkUtil.WSysAqua) \
+            and RO.TkUtil.getTclVersion().startswith("8.5"):
+            if showIndicator:
+                corrWidth = width + 3
+            else:
+                corrWidth = width + 2
+            return corrWidth
+        return width
 
 
 if __name__ == "__main__":
     import PythonTk
+    from StatusBar import StatusBar
     root = PythonTk.PythonTk()
     
     def btnCallback(btn):
         print "%s state=%s" % (btn["text"], btn.getBool())
 
+    row = 0
+    col = 0
     Checkbutton(root,
-        text = "Auto Bgnd",
+        text = "Auto Bgnd A",
         defValue = False,
         callFunc = btnCallback,
-        helpText = "Help for checkbox 'Auto Bgnd', default off",
+        helpText = "defValue=False, autoIsCurrent=True",
         autoIsCurrent = True,
-    ).pack()
+    ).grid(row=row, column=col, sticky="w")
+    row += 1
     Checkbutton(root,
-        text = "Manual",
+        text = "Auto Bgnd B",
         defValue = True,
         callFunc = btnCallback,
-        helpText = "Help for checkbox 'Manual', default on",
-    ).pack()
-    var = Tkinter.StringVar()
-    Checkbutton(root,
-        text = "Auto Bgnd 2",
-        defValue = True,
-        callFunc = btnCallback,
-        helpText = "Help for checkbox 'Auto Bgnd 2', default on",
+        helpText = "defValue=True, autoIsCurrent=True, indicatoron=False",
         indicatoron = False,
         autoIsCurrent = True,
-    ).pack()
+    ).grid(row=row, column=col, sticky="w")
+    row = 0
+    col += 1
     Checkbutton(root,
-        text = "Manual 2",
+        text = "MmmmmNnnnn A",
+        width = 12,
         defValue = True,
         callFunc = btnCallback,
-        helpText = "Help for checkbox 'Manual 2', default on",
+        helpText = "width=12, defValue=True",
+    ).grid(row=row, column=col, sticky="w")
+    row += 1
+    Checkbutton(root,
+        text = "MmmmmNnnnn B",
         indicatoron = False,
-    ).pack()
+        width = 12,
+        defValue = True,
+        callFunc = btnCallback,
+        helpText = "width=12, defValue=True, indicatoron=False",
+    ).grid(row=row, column=col, sticky="w")
+    row = 0
+    col += 1
+    Checkbutton(root,
+        text = "MmmmmNnnnn C",
+        width = 12,
+        defValue = True,
+        callFunc = btnCallback,
+        helpText = "defValue=True",
+    ).grid(row=row, column=col, sticky="w")
+    row += 1
+    Checkbutton(root,
+        text = "MmmmmNnnnn D",
+        indicatoron = False,
+        width = 12,
+        defValue = True,
+        callFunc = btnCallback,
+        helpText = "defValue=True, indicatoron=False",
+    ).grid(row=row, column=col, sticky="w")
+    row += 1
+    StatusBar(root).grid(row=row, column=0, columnspan=3, sticky="ew")
 
     root.mainloop()
