@@ -98,6 +98,9 @@ History:
 2012-11-30 ROwen    Bug fix: width patch was not applied if width changed after the widget was created.
                     Now it is applied by overridden method configure.
 2012-11-30 ROwen    Does no width correction if bitmap is shown.
+2014-02-04 ROwen    Improve label handling: the widget width was affected by the current value,
+                    rather than the width of the label, and label="" was treated as None.
+2014-02-10 ROwen    Added forceValid argument to set.
 """
 __all__ = ['OptionMenu']
 
@@ -140,7 +143,8 @@ class OptionMenu (Tkinter.Menubutton, RO.AddCallback.TkVarMixin,
     - callFunc  callback function (not called when added);
                 the callback receives one argument: this object
     - defMenu   name of "restore default" contextual menu item, or None for none
-    - label     label for menu; if omitted then the label is automatically set to the selected value.
+    - label     label for menu; if None then the label is automatically set to the selected value.
+                Use "" to always display an empty menu.
     - abbrevOK  controls the behavior of set and setDefault;
                 if True then unique abbreviations are allowed
     - ignoreCase controls the behavior of set and setDefault;
@@ -182,6 +186,7 @@ class OptionMenu (Tkinter.Menubutton, RO.AddCallback.TkVarMixin,
         showDefault = not (var and defValue == None)
         if var == None:
             var = Tkinter.StringVar()
+        self._tempValue = None
         self._items = []
         self.defValue = None
         self.noneDisplay = noneDisplay or ''
@@ -209,7 +214,7 @@ class OptionMenu (Tkinter.Menubutton, RO.AddCallback.TkVarMixin,
         for item in ("text", "textvariable"):
             if wdgKArgs.has_key(item):
                 del(wdgKArgs[item])
-        if label:
+        if label is not None:
             wdgKArgs["text"] = label
         else:
             wdgKArgs["textvariable"] = var
@@ -256,7 +261,7 @@ class OptionMenu (Tkinter.Menubutton, RO.AddCallback.TkVarMixin,
         """
         if argDict is not None:
             kargs.update(argDict)
-        if "width" in kargs:
+        if self.label is None and "width" in kargs:
             kargs["width"] = self._computeCorrectedWidth(
                 width = kargs["width"],
                 hasBitmap = bool(kargs.get("bitmap", self["bitmap"])),
@@ -294,6 +299,7 @@ class OptionMenu (Tkinter.Menubutton, RO.AddCallback.TkVarMixin,
             if descr and value != None:
                 menuText = "%s (%s)" % (descr, value)
                 def setValue():
+                    self._tempValue = None
                     self.set(value)
                 menu.add_command(label = menuText, command = setValue)
 
@@ -417,7 +423,8 @@ class OptionMenu (Tkinter.Menubutton, RO.AddCallback.TkVarMixin,
     def isValid(self):
         """Return True if the currently displayed value is one of the items set by setItems
         """
-        return self._var.get() in self._items
+        return self._var.get() in self._items \
+            or (self._tempValue is not None and self._var.get() == self._tempValue)
     
     def restoreDefault(self):
         """Restore the default value.
@@ -425,12 +432,24 @@ class OptionMenu (Tkinter.Menubutton, RO.AddCallback.TkVarMixin,
         #print "restoreDefault(); currValue=%r, defValue=%r" % (self._var.get(), self.defValue,)
         self._var.set(self.asString(self.defValue))
 
-    def set(self, newValue, isCurrent=True, doCheck=True, *args, **kargs):
+    def set(self, newValue, isCurrent=True, doCheck=True, forceValid=False, *args, **kargs):
         """Changes the currently selected value.
+
+        Inputs:
+        - newValue: new value to set
+        - isCurrent: is the new value current?
+        - doCheck: test if the new value is one of the allowed items? Ignored if forceValid is true
+        - forceValid: the new value is forced to be valid. The value is cleared
+            when the user selects a menu item or set is called again.
+        *args, **kargs: ignored
         """
+        self._tempValue = None
         newValue, isOK = self.expandValue(newValue)
-        if not isOK and doCheck:
-            raise ValueError("Value %r invalid" % newValue)
+        if not isOK:
+            if forceValid:
+                self._tempValue = str(newValue)
+            elif doCheck:
+                raise ValueError("Value %r invalid" % newValue)
     
         self.setIsCurrent(isCurrent)
         self._var.set(self.asString(newValue))
@@ -587,8 +606,9 @@ class OptionMenu (Tkinter.Menubutton, RO.AddCallback.TkVarMixin,
         
         Only register this callback function on aqua Tcl/Tk 8.5
         """
-        currVal = self._var.get()
-        self["width"] = len(currVal)
+        if self.label is None:
+            currVal = self._var.get()
+            self["width"] = len(currVal)
 
 
 if __name__ == "__main__":
